import java.io.*;
// Readline is the library that helps us with input.
import org.gnu.readline.*;
// jtux is the library that gives the POSIX interface.
import jtux.*;

public class Shell {

    static public void main(String[] args) {

	// First some readline stuff. You can safely ignore this:
	if (args.length > 1)
	    Readline.load(ReadlineLibrary.byName(args[1]));
	else
	    Readline.load(ReadlineLibrary.GnuReadline);
	
	Readline.initReadline("Shell"); // init, set app name, read inputrc

	try {
	    if (args.length > 0)
		Readline.readInitFile(args[0]);    // read private inputrc
	} catch (IOException e) {              // this deletes any initialization
	    System.out.println(e.toString());    // from /etc/inputrc and ~/.inputrc
	    System.exit(0);
	}

	// read history file, if available
	java.io.File history = new java.io.File(".osshell_history");
	try {
	    if (history.exists())
		Readline.readHistoryFile(history.getName());
	} catch (Exception e) {
	    System.err.println("Error reading history file!");
	}

	// Set word break characters
	try {
	    Readline.setWordBreakCharacters(" \t;");
	}
	catch (UnsupportedEncodingException enc) {
	    System.err.println("Could not set word break characters");
	    System.exit(0);
	}

	// main input loop. You should understand everything from this point on.

	String line; // This will contain whatever we type at the prompt.
	boolean exit = false; // Should we exit the main loop?
	String[] commands; // When we give more than one command, we need to
	// split line up. The first command in line will be command[0], the second
	// will be command[1], etc. You will not need to use this variable in the
	// assignment.
	while (!exit) {
	    try {
		// First we get the input line:
		line = Readline.readline("Da Prompt> ");
		if (line == null) {
		    System.out.println("Say what?");
		} else {
		    // Now that we have the input we need to split it into several,
		    // if more than one was given. We use | as a delimiter:
		    commands = line.split("\\|");
		    // We also need to separate the actual command from its arguments.
		    // The first argument is always the command name.
		    String[][] arguments = new String[commands.length][];
		    for (int i=0; i<commands.length;i++) {
			commands[i] = commands[i].trim();
			arguments[i] = commands[i].split("\\s");
		    }
		    // When we've split the line we also trim the different commands
		    // from any unneeded whitespace:
		    for (int i=0; i < commands.length; i++)
			arguments[i][0] = arguments[i][0].trim();
		    // If the user typed exit, let's exit:
		    if (arguments[0][0].compareTo("exit") == 0)
			exit = true;
		    else
			// At this point the arguments array will be ready to use.
			// Here is an example of what it looks like:
			// If you type
			//
			// ls -lF | wc -lw
			//
			// at the command line, the array "arguments" will look like
			//
			// {{"ls", "-lF"},
			//  {"wc", "-lw"}}
			//
			// So, arguments[0] would be {"ls", "-lF"}, and arguments[1]
			// would be {"wc", "-lw"}. Further, arguments[0][0] would be
			// "ls", arguments[0][1] would be "-lF", arguments[1][0] would
			// be "wc", and arguments[1][1] would be "-lw".
		    try {
			// Here you should do most work. Right now we just
			// echo the commands given.
			for (int i=0;i<arguments.length;i++) {
			    System.out.print("Command: " + arguments[i][0] + ", arguments: ");
			    for (int j=0;j<arguments[i].length;j++)
				System.out.print(arguments[i][j] + " ");
			    System.out.println(" ");
			}
		    } catch(Exception e) {
			System.out.print("Exception: ");
			System.out.println(">" + e.toString() + "<");
		    }
		}
	    } catch (UnsupportedEncodingException enc) {
		System.err.println("caught UnsupportedEncodingException");
		break;
	    } catch (IOException eof) {
		break;
	    }
	}
	try {
	    Readline.writeHistoryFile(history.getName());
	} catch (Exception e) {
	    System.err.println("Error writing history file!");
	}
	Readline.cleanup();
	System.exit(0);
    }
}
